
// LoadAdjList.java
// Dimitrios Michail and Contributors, 2021
// Modified by Andrew Davison, ad@fivedots.coe.psu.ac.th, Jan. 2023

/* Read data from a text file, build the graph, display it, 
   then save it to a 'Copy' file, which is then read back in
   and redisplayed.

   There are two method to test the unweighted and weighted 
   read methods: JGTUtils.readAdjList() and 
   JGTUtils.readWAdjList().

   The unweighted file is adjlist1.txt; 
   the weighted file is adjlist2.txt (both are directed graphs)

   The weighted graph uses my WeightedEdge class for the edges
   which assumes the weights are integers.

   For related code, see LoadAdjMatrix.java, and LoadGraphML.java

  Usage:
    > compile LoadAdjList.java
    > run LoadAdjList
*/

import java.io.*;
import java.util.*;

import org.jgrapht.*;
import org.jgrapht.graph.*;



public class LoadAdjList
{

  public static void main(String[] args)
  {
    unweighted();
    weighted();
  }  // end of main()



  private static void unweighted()
  // read, display, save, read back in, display
  {
    System.out.println("----------- unweighted -----------------\n");

    // read and build
    String input = JGTUtils.loadText("adjlist1.txt");
    Graph<String, DefaultEdge> g =
      JGTUtils.readAdjList(input, true);  
                          // the graph is directed
    // display
    System.out.println("\nAdjlist 1 graph:\n" + g);
    // JGTUtils.displayAsDOT(g);
    JGTUtils.displayAsMX("unweighted", g);

    // save
    String gStr = JGTUtils.writeAdjList(g);
    System.out.println("\nAdj list version:\n" +  gStr);
    JGTUtils.saveText("adjlist1Copy.txt", gStr);

    // read in again
    System.out.println();
    input = JGTUtils.loadText("adjlist1Copy.txt");

    g = JGTUtils.readAdjList(input, true);  // directed

    System.out.println("\nAdjlist 1 Copy graph:\n" + g);
    JGTUtils.displayAsMX("unweighted copy", g);
  }  // end of unweighted()



  private static void weighted()
  // read, display, save, read back in, display
  {
    System.out.println("\n----------- weighted -----------------\n");

    // read and build
    String input = JGTUtils.loadText("adjlist2.txt");
    Graph<String, WeightedEdge> g =
      JGTUtils.readWAdjList(input, true); 
                          // the graph is directed
    // display
    System.out.println("\nAdjlist 2 graph:\n" + g);
    System.out.println("W A->B: " + g.getEdgeWeight(g.getEdge("A", "B")));
    System.out.println("W A->C: " + g.getEdgeWeight(g.getEdge("A", "C")));
    System.out.println("W A->D: " + g.getEdgeWeight(g.getEdge("A", "D")));
    System.out.println("W B->C: " + g.getEdgeWeight(g.getEdge("B", "C")));
    // JGTUtils.displayAsDOT(g);
    JGTUtils.displayAsMX("weighted", g);

    // save
    String gStr = JGTUtils.writeAdjList(g);
    System.out.println("\nAdj list version:\n" +  gStr);
    JGTUtils.saveText("adjlist2Copy.txt", gStr);

    // read in again
    System.out.println();
    input = JGTUtils.loadText("adjlist2Copy.txt");

    g = JGTUtils.readWAdjList(input, true);  // directed

    System.out.println("\nAdjlist 2 Copy graph:\n" + g);
    JGTUtils.displayAsMX("weighted copy", g);
    System.out.println("W A->B: " + g.getEdgeWeight(g.getEdge("A", "B")));
  }  // end of weighted()


}  // end of LoadAdjList class
