
// LoadGraphML.java
// Dimitrios Michail and Contributors, 2021
// Modified by Andrew Davison, ad@fivedots.coe.psu.ac.th, Jan. 2023

/* Read data from a text file, build the graph, display it, 
   then save it to a 'Copy' file, which is then read back in
   and redisplayed.

   There are two method to test the unweighted and weighted 
   read methods: JGTUtils.readGraphML() and 
   JGTUtils.readWGraphML().

   The unweighted file is g1.graphml; 
   the weighted file is g2.graphml (both are directed graphs)

   The weighted graph uses my WeightedEdge class for the edges
   which assumes the weights are integers.

   For more info on GraphML, see:
    * http://graphml.graphdrawing.org/
    * https://en.wikipedia.org/wiki/GraphML

   For related code, see LoadAdjList.java, and LoadAdjMatrix.java

  Usage:
    > compile LoadGraphML.java
    > run LoadGraphML
*/

import java.io.*;
import java.util.*;

import org.jgrapht.*;
import org.jgrapht.graph.*;


public class LoadGraphML
{

  public static void main(String[] args)
  {
    unweighted();
    weighted();
  }  // end of main()



  private static void unweighted()
  // read, display, save, read back in, display
  {
    System.out.println("----------- unweighted -----------------\n");

    // read and build
    String input = JGTUtils.loadText("g1.graphml");
    Graph<String, DefaultEdge> g = 
                 JGTUtils.readGraphML(input, true); 
                               // the graph is directed
    // display
    System.out.println("\nGraphML 1 graph:\n" + g);
    // JGTUtils.displayAsDOT(g);
    JGTUtils.displayAsMX("unweighted", g);

    // save
    String gStr = JGTUtils.writeGraphML(g);
    System.out.println("\nGraphML version:\n" + gStr);
    JGTUtils.saveText("g1Copy.graphml", gStr);

    // read in again
    System.out.println();
    input = JGTUtils.loadText("g1Copy.graphml");
    g = JGTUtils.readGraphML(input, true);  // directed

    System.out.println("\nGraphML 1 Copy graph:\n" + g);
    JGTUtils.displayAsMX("unweighted copy", g);
  } // end of unweighted()



  private static void weighted()
  // read, display, save, read back in, display
  {
    System.out.println("\n----------- weighted -----------------\n");

    // read and build
    String input = JGTUtils.loadText("g2.graphml");
    // input = JGTUtils.loadText("g1.graphml");
    Graph<String, WeightedEdge> g = 
                 JGTUtils.readWGraphML(input, true);
                               // the graph is directed
    // display
    System.out.println("\nGraphML 2 graph:\n" + g);
    System.out.println("W 1->2: " + g.getEdgeWeight(g.getEdge("1", "2")));
    System.out.println("W 2->3: " + g.getEdgeWeight(g.getEdge("2", "3")));
    // JGTUtils.displayAsDOT(g);
    JGTUtils.displayAsMX("weighted", g);

    // save
    String gStr = JGTUtils.writeGraphML(g);
    // System.out.println("\nGraphML version:\n" + gStr);    // long
    JGTUtils.saveText("g2Copy.graphml", gStr);

    // read in again
    System.out.println();
    input = JGTUtils.loadText("g2Copy.graphml");
    g = JGTUtils.readWGraphML(input, true);  // directed

    System.out.println("\nGraphML 2 Copy graph:\n" + g);
    JGTUtils.displayAsMX("weighted copy", g);

    System.out.println("W 1->2: " + g.getEdgeWeight(g.getEdge("1", "2")));
    System.out.println("W 2->3: " + g.getEdgeWeight(g.getEdge("2", "3")));
  } // end of weighted()


}  // end of LoadGraphML class
